using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net.Interfaces;
using Luski.net.JsonTypes;
using File = Luski.net.JsonTypes.File;

namespace Luski.GUI.MainScreen.UI;

public class ChatMessage : UserControl
{
    readonly int padding = 2;
    private static Font TimeFont = Font.MakeFontFromSystem(13);
    private SocketMessage Msg { get; }
    private Label label1, label2, lastm;

    public ChatMessage(SocketMessage message)
    {
        Size = new(1467, 74);
        BackgroundColor = new(40, 40, 40, 255);
        Msg = message;
        IUser user = message.GetAuthor().Result;
        Anchor = ObjectAnchor.Left | ObjectAnchor.Right;
        
        Controls.Add(label1 = new Label() {Location = new(83, 9), Text = user.Username});
        
        
        
        DateTime time = new DateTime(2022, 1, 1, 0, 0, 0, 0).AddMilliseconds(Msg.Id >> 22).ToLocalTime();
        string timestr;
        if (time.Date == DateTime.Now.ToLocalTime().Date)
        {
            timestr = $"Today at {time.ToShortTimeString()}";
        }
        else if (time.Date == DateTime.Now.ToLocalTime().AddDays(-1).Date)
        {
            timestr = $"Yesterday at {time.ToShortTimeString()}";
        }
        else
        {
            timestr = $"{time:M/dd/yyyy h:mm tt}";
        }
        Controls.Add(new Label() {Location = new(label1.Location.X + label1.Size.X + 4, 9), Text = timestr});
        Controls.Add(new Rectangle(new Texture(user.GetAvatar().Result)) { Location = new(15, 3), Size = new(58, 58) });
        Controls.Add(label2 = new Label() {Location = new(83, 40), Text = message.Context});
        lastm = label2;
        if (Msg.Files != null && Msg.Files.Length > 0)
        {
            int row = 1;
            int filesonrow = 0;
            for (int i = 0; i < Msg.Files.Length; i++)
            {
                int lx = (padding * filesonrow) + lastm.Location.X + (333 * (filesonrow + 1));
                if (lx > Size.X)
                {
                    row++;
                    filesonrow = 0;
                    lx = (padding * filesonrow) + lastm.Location.X + (333 * (filesonrow + 1));
                }

                filesonrow++;
                Controls.Add(new ContentEmbed(Msg.Files[i], Msg.ChannelID)
                {
                    Location = new(lx - 333,  lastm.Location.Y + 2 + lastm.Size.Y +(padding * row) + (66 * (row - 1)))
                });
            }
        }

        Size = new(Size.X, Controls[Controls.Length - 1].Location.Y + Controls[Controls.Length - 1].Size.Y + padding + 10);
    }
    
    public void AddMessage(SocketMessage msg)
    {
        Label newLabel = new()
        {
            Text = msg.Context,
            Tag = msg,
            Location = new(label2.Location.X, Size.Y)
        };

        newLabel.MouseEnter += NewLabel_MouseEnter;
        newLabel.MouseLeave += NewLabel_MouseLeave;
        Controls.Add(newLabel);
        if (msg.Files != null && msg.Files.Length > 0)
        {
            int row = 1;
            int filesonrow = 0;
            for (int i = 0; i < Msg.Files.Length; i++)
            {
                int lx = (padding * filesonrow) + lastm.Location.X + (333 * (filesonrow + 1));
                if (lx > Size.X)
                {
                    row++;
                    filesonrow = 0;
                    lx = (padding * filesonrow) + lastm.Location.X + (333 * (filesonrow + 1));
                }

                filesonrow++;
                Controls.Add(new ContentEmbed(Msg.Files[i], Msg.ChannelID)
                {
                    Location = new(lx - 333,  lastm.Location.Y + 2 + lastm.Size.Y +(padding * row) + (66 * (row - 1)))
                });
            }
        }
        Size = new(Size.X, Controls[Controls.Length - 1].Location.Y + Controls[Controls.Length - 1].Size.Y + padding + 10);
    }
    readonly List<Label> Labels = new();
    private Task NewLabel_MouseLeave(IRenderObject sender)
    {
        if (sender is not Label label) return Task.CompletedTask;
        if (label.Tag is not SocketMessage Message) return Task.CompletedTask;
        DateTime time = new DateTime(2022, 1, 1, 0, 0, 0, 0).AddMilliseconds(Message.Id >> 22).ToLocalTime();
        string b = time.ToString("h:mm tt");
        IEnumerable<Label>? l = Labels.Where(s => s.Text == b);
        if (l is not null && l.Any())
        {
            Controls.Remove(l.First());
            Labels.Remove(l.First());
        }
        Window!.DrawFrame();
        return Task.CompletedTask;
    }
    
    private Task NewLabel_MouseEnter(IRenderObject sender)
    {
        if (sender is not Label label) return Task.CompletedTask;
        if (label.Tag is not SocketMessage Message) return Task.CompletedTask;
        DateTime time = new DateTime(2022, 1, 1, 0, 0, 0, 0).AddMilliseconds(Message.Id >> 22).ToLocalTime();
        Label m = new()
        {
            Text = time.ToString("h:mm tt"),
            Location = new(15, label.Location.Y - (int)TimeFont.PixelHeight + (int)label.Font.PixelHeight),
            Font = TimeFont
        };
        
        Controls.Add(m);
        Labels.Add(m);
        Window!.DrawFrame();
        return Task.CompletedTask;
    }
}