using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net.Enums;
using Luski.net.JsonTypes;
using OpenTK.Mathematics;

namespace Luski.GUI.MainScreen.UI;

public class FriendRequest : UserControl
{
    private MainScreen Screen;
    public SocketRemoteUser User;
    public Rectangle Accept, Reject;
    
    public FriendRequest(MainScreen Parent, SocketRemoteUser User)
    {
        this.User = User;
        this.Screen = Parent;
        Size = new(400, 96);
        BackgroundColor = new(34, 34, 34, 255);
        Controls.Add(new Rectangle(new Texture(User.GetAvatar().Result)) { Location = new(15,17), Size = new(58,58)});
        Controls.Add(new Label() { Text = User.Username, Location = new(77,20)});
        Controls.Add(new Label() { Text = User.FriendStatus.ToString(), Location = new(77,48)});
        if (User.FriendStatus == FriendStatus.PendingIn)
        {
            Controls.Add(Accept = new Rectangle()
            {
                Anchor = ObjectAnchor.Top | ObjectAnchor.Right, BackgroundColor = Color4.Green, Location = new(254, 17),
                Size = new(58, 58)
            });
            Controls.Add(Reject = new Rectangle()
            {
                Anchor = ObjectAnchor.Top | ObjectAnchor.Right, BackgroundColor = Color4.Red, Location = new(327, 17),
                Size = new(58, 58)
            });
            Accept.Clicked += AcceptOnClicked;
            Reject.Clicked += RejectOnClicked;
        }
    }

    private Task RejectOnClicked(IRenderObject arg)
    {
        _ = Globals.Luski.SendFriendResult(User.Id, false).Result;
        Screen.RemoveFriendRequest(User);
        return Task.CompletedTask;
    }

    private Task AcceptOnClicked(IRenderObject arg)
    {
        Screen.AddFriend(Globals.Luski.SendFriendResult(User.Id, true).Result);
        Screen.RemoveFriendRequest(User);
        return Task.CompletedTask;
    }
}